import 'package:flutter/material.dart';
import 'package:unify_api/unify.dart';
import 'package:unify_api_example/common/index.dart';
import 'package:unify_api_example/utils/index.dart';

class AccountInfoExample extends StatefulWidget {
  final UnifyClient client;

  const AccountInfoExample({Key key, this.client}) : super(key: key);

  @override
  _AccountInfoExampleState createState() => _AccountInfoExampleState();
}

class _AccountInfoExampleState extends State<AccountInfoExample> {
  AccountInfo _details;
  @override
  void initState() {
    widget.client.getInfo().then((details) {
      setState(() {
        _details = details;
      });
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Account Details"),
        actions: <Widget>[
          JsonButton(json: _details?.json),
        ],
      ),
      body: Builder(
        builder: (BuildContext context) {
          if (_details == null)
            return Center(child: CircularProgressIndicator());
          return ListView(
            children: <Widget>[
              ListTile(
                title: Text("Full Name"),
                subtitle: Text(_details.fullName),
              ),
              ListTile(
                title: Text("Video Enabled"),
                subtitle: Text(_details.videoEnabled.toString()),
              ),
              ListTile(
                title: Text("Email"),
                subtitle: Text(_details.email),
              ),
              ListTile(
                title: Text("Primary Phone"),
                subtitle: Text(_details.primaryPhone.toString()),
              ),
              ListTile(
                title: Text("Secondary Phone"),
                subtitle: Text(_details.secondaryPhone.toString()),
              ),
              ListTile(
                title: Text("Access Token"),
                subtitle: TextFormField(
                    initialValue: _details.client.token.accessToken),
              ),
              ListTile(
                title: Text("Unify Alerts"),
                subtitle: Text("Tap To View"),
                onTap: () async {
                  var items = await widget.client.listUnifyAlerts() ?? [];

                  List<Widget> _children = [];
                  _children = items
                      .map((i) => ListTile(
                            title: Text(i.type),
                            subtitle: Text(i.date.toString()),
                            trailing: JsonButton(json: i?.json),
                          ))
                      .toList();
                  showList(context, _children);
                },
              ),
            ],
          );
        },
      ),
    );
  }
}
