import 'package:flutter/material.dart';
import 'package:unify_api/unify.dart';
import 'package:unify_api_example/common/index.dart';
import 'package:unify_api_example/ui/contacts.dart';

class ContactGroupsExample extends StatefulWidget {
  final UnifyClient client;

  const ContactGroupsExample({Key key, this.client}) : super(key: key);
  @override
  _ContactGroupsExampleState createState() => _ContactGroupsExampleState();
}

class _ContactGroupsExampleState extends State<ContactGroupsExample> {
  List<ContactGroup> _items;

  @override
  void initState() {
    widget.client.listContactGroups().then((items) {
      setState(() {
        _items = items;
      });
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Contact Groups"),
        actions: <Widget>[
          IconButton(
            icon: Icon(Icons.add),
            onPressed: () {
              Navigator.push<String>(
                  context,
                  MaterialPageRoute(
                    builder: (BuildContext context) {
                      return new AddGroupExample();
                    },
                    fullscreenDialog: true,
                  )).then((value) {
                if (value != null) {
                  widget.client.addContactGroup(value).catchError((error) {
                    print("Error Adding Group: $error");
                  });
                }
              });
            },
          ),
        ],
      ),
      body: Builder(
        builder: (context) {
          if (_items == null) return Center(child: CircularProgressIndicator());
          if (_items.isEmpty)
            return Center(child: Text("No Contact Groups Found"));
          return ListView.builder(
            itemBuilder: (BuildContext context, int index) {
              final i = _items[index];
              return ListTile(
                title: Text(i.name),
                subtitle: Text("Count: ${i.count}"),
                trailing: JsonButton(json: i?.json),
                onTap: () {
                  Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) => ContactsInGroupExample(
                        client: widget.client,
                        group: i,
                      ),
                    ),
                  );
                },
              );
            },
            itemCount: _items?.length ?? 0,
          );
        },
      ),
    );
  }
}

class ContactsInGroupExample extends StatefulWidget {
  final UnifyClient client;
  final ContactGroup group;

  const ContactsInGroupExample({Key key, this.client, @required this.group})
      : super(key: key);
  @override
  _ContactsInGroupExampleState createState() => _ContactsInGroupExampleState();
}

class _ContactsInGroupExampleState extends State<ContactsInGroupExample> {
  List<Contact> _items;
  List<String> _emails;

  @override
  void initState() {
    widget.group.listContacts().then((items) {
      setState(() {
        _items = items;
      });
    });
    widget.group.listEmails().then((items) {
      setState(() {
        _emails = items;
      });
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return DefaultTabController(
        length: 2,
        initialIndex: 0,
        child: Scaffold(
          appBar: AppBar(
            title: Text(widget?.group?.name ?? ""),
            actions: <Widget>[
              IconButton(
                icon: Icon(Icons.delete),
                onPressed: () {
                  widget.group.delete().then((_) {
                    Navigator.pop(context);
                  }).catchError((error) {
                    print("Error: $error");
                  });
                },
              ),
              IconButton(
                icon: Icon(Icons.edit),
                onPressed: () {
                  Navigator.push<String>(
                      context,
                      MaterialPageRoute(
                        builder: (BuildContext context) {
                          return new AddGroupExample(
                            name: widget.group.name,
                          );
                        },
                        fullscreenDialog: true,
                      )).then((value) {
                    if (value != null) {
                      widget.group.edit(value).catchError((error) {
                        print("Error Editing Group: $error");
                      });
                    }
                  });
                },
              ),
              JsonButton(json: widget?.group?.json)
            ],
            bottom: TabBar(
              tabs: <Widget>[
                Tab(text: "Contacts"),
                Tab(text: "Emails"),
              ],
            ),
          ),
          body: TabBarView(
            children: <Widget>[
              Builder(
                builder: (BuildContext context) {
                  if (_items == null)
                    return Center(child: CircularProgressIndicator());
                  if (_items.isEmpty)
                    return Center(child: Text("No Contacts Found"));
                  return ListView.builder(
                    itemBuilder: (BuildContext context, int index) {
                      final i = _items[index];
                      return ListTile(
                        title: Text(i.displayName),
                        subtitle: Text(i.lastActivity),
                        trailing: JsonButton(json: i?.json),
                        onTap: () {
                          Navigator.push(
                            context,
                            MaterialPageRoute(
                              builder: (BuildContext context) =>
                                  ContactDetailsExample(
                                      contact: i.getDetails()),
                            ),
                          );
                        },
                      );
                    },
                    itemCount: _items?.length ?? 0,
                  );
                },
              ),
              Builder(
                builder: (BuildContext context) {
                  if (_emails == null)
                    return Center(child: CircularProgressIndicator());
                  if (_emails.isEmpty)
                    return Center(child: Text("No Emails Found"));
                  return ListView.builder(
                    itemBuilder: (BuildContext context, int index) {
                      final i = _emails[index];
                      return ListTile(
                        title: Text(i),
                      );
                    },
                    itemCount: _emails?.length ?? 0,
                  );
                },
              ),
            ],
          ),
        ));
  }
}
