import 'package:flutter/material.dart';
import 'package:unify_api/unify.dart';
import 'package:unify_api_example/common/index.dart';
import 'package:unify_api_example/utils/index.dart';

class LeadsExample extends StatefulWidget {
  final UnifyClient client;

  const LeadsExample({Key key, this.client}) : super(key: key);
  @override
  _LeadsExampleState createState() => _LeadsExampleState();
}

class _LeadsExampleState extends State<LeadsExample> {
  List<Lead> _items;

  @override
  void initState() {
    widget.client.listLeads().then((items) {
      setState(() {
        _items = items;
      });
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Leads"),
        actions: <Widget>[
          IconButton(
            icon: Icon(Icons.add),
            onPressed: () {
              Navigator.push<LeadDetails>(
                  context,
                  MaterialPageRoute(
                    builder: (BuildContext context) {
                      return new AddLeadExample(
                        client: widget.client,
                      );
                    },
                    fullscreenDialog: true,
                  )).then((value) {
                if (value != null) {
                  LeadDetails _info = value;
                  widget.client.addLead(
                    firstName: _info?.firstName,
                    lastName: _info?.lastName,
                    middleName: _info?.middleName,
                    suffix: _info?.suffix,
                    secondaryFirstName: _info?.secondaryFirstName,
                    secondaryMiddleName: _info?.secondaryMiddleName,
                    secondaryLastName: _info?.secondaryLastName,
                    currentAddress: _info?.currentAddress,
                    cellPhone: _info?.cellPhone,
                    homePhone: _info?.homePhone,
                    officePhone: _info?.officePhone,
                    faxPhone: _info?.faxPhone,
                    secondaryCellPhone: _info?.secondaryCellPhone,
                    email: _info?.email,
                    groups: _info?.groups,
                    bestCallTime: _info?.bestCallTime,
                    creditRange: _info?.creditRange,
                    secondaryCreditRange: _info?.secondaryCreditRange,
                    annualGrossIncome: _info?.annualGrossIncome,
                    recentBankruptcy: _info?.recentBankruptcy,
                    propertyValue: _info?.propertyValue,
                    propertyAddress: _info?.propertyAddress,
                  );
                }
              });
            },
          ),
        ],
      ),
      body: Builder(
        builder: (BuildContext context) {
          if (_items == null) return Center(child: CircularProgressIndicator());
          if (_items.isEmpty) return Center(child: Text("No Leads Found"));
          return ListView.builder(
            itemBuilder: (BuildContext context, int index) {
              final i = _items[index];
              return ListTile(
                title: Text(i.displayName),
                subtitle: Text(i.lastActivity),
                trailing: JsonButton(json: i?.json),
                onTap: () {
                  Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) =>
                          LeadDetailsExample(lead: i.getDetails()),
                    ),
                  );
                },
              );
            },
            itemCount: _items?.length ?? 0,
          );
        },
      ),
    );
  }
}

class LeadDetailsExample extends StatefulWidget {
  final Future<LeadDetails> lead;

  const LeadDetailsExample({Key key, @required this.lead}) : super(key: key);
  @override
  _LeadDetailsExampleState createState() => _LeadDetailsExampleState();
}

class _LeadDetailsExampleState extends State<LeadDetailsExample> {
  LeadDetails _details;

  @override
  void initState() {
    widget.lead.then((details) {
      setState(() {
        _details = details;
      });
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Lead Details"),
        actions: <Widget>[
          IconButton(
            icon: Icon(Icons.delete),
            onPressed: () {
              _details.delete().then((_) {
                Navigator.pop(context);
              }).catchError((error) {
                print("Error: $error");
              });
            },
          ),
          IconButton(
            icon: Icon(Icons.edit),
            onPressed: () {
              Navigator.push<LeadDetails>(
                  context,
                  MaterialPageRoute(
                    builder: (BuildContext context) {
                      return new AddLeadExample(
                        client: _details.client,
                        lead: _details,
                      );
                    },
                    fullscreenDialog: true,
                  )).then((value) {
                if (value != null) {
                  LeadDetails _info = value;
                  _details.edit(
                    firstName: _info?.firstName,
                    lastName: _info?.lastName,
                    middleName: _info?.middleName,
                    suffix: _info?.suffix,
                    secondaryFirstName: _info?.secondaryFirstName,
                    secondaryMiddleName: _info?.secondaryMiddleName,
                    secondaryLastName: _info?.secondaryLastName,
                    currentAddress: _info?.currentAddress,
                    cellPhone: _info?.cellPhone,
                    homePhone: _info?.homePhone,
                    officePhone: _info?.officePhone,
                    faxPhone: _info?.faxPhone,
                    secondaryCellPhone: _info?.secondaryCellPhone,
                    email: _info?.email,
                    groups: _info?.groups,
                    bestCallTime: _info?.bestCallTime,
                    creditRange: _info?.creditRange,
                    secondaryCreditRange: _info?.secondaryCreditRange,
                    annualGrossIncome: _info?.annualGrossIncome,
                    recentBankruptcy: _info?.recentBankruptcy,
                    propertyValue: _info?.propertyValue,
                    propertyAddress: _info?.propertyAddress,
                  );
                }
              });
            },
          ),
          JsonButton(json: _details?.json),
        ],
      ),
      body: FutureBuilder<LeadDetails>(
        initialData: _details,
        future: widget.lead,
        builder: (BuildContext context, AsyncSnapshot<LeadDetails> snapshot) {
          if (snapshot?.data == null)
            return Center(child: CircularProgressIndicator());

          LeadDetails details = snapshot.data;

          return ListView(
            children: <Widget>[
              ListTile(
                title: Text("Full Name"),
                subtitle: Text(details.fullName),
              ),
              ListTile(
                title: Text("Email"),
                subtitle: Text(details.email),
              ),
              ListTile(
                title: Text("Cell Phone"),
                subtitle: Text(details.cellPhone.toString()),
              ),
              ListTile(
                title: Text("Home Phone"),
                subtitle: Text(details.homePhone.toString()),
              ),
              ListTile(
                title: Text("Office Phone"),
                subtitle: Text(details.officePhone.toString()),
              ),
              ListTile(
                title: Text("Notes"),
                subtitle: Text("Tap To View"),
                trailing: IconButton(
                  icon: Icon(Icons.note_add),
                  onPressed: () {
                    Navigator.push<String>(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) {
                            return new AddNoteExample();
                          },
                          fullscreenDialog: true,
                        )).then((value) {
                      if (value != null) {
                        details.addNote(value);
                      }
                    });
                  },
                ),
                onTap: () {
                  var items = details?.notes ?? [];
                  List<Widget> _children = [];
                  _children = items
                      .map((i) => ListTile(
                            title: Text(i.note),
                          ))
                      .toList();
                  showList(context, _children);
                },
              ),
              ListTile(
                title: Text("Tracker"),
                subtitle: Text("Tap To View"),
                trailing: IconButton(
                    icon: Icon(Icons.timer),
                    onPressed: () {
                      Navigator.push<Map<String, dynamic>>(
                          context,
                          MaterialPageRoute(
                            builder: (BuildContext context) {
                              return new AddLogResponseExample(
                                client: details.client,
                                responses: details.listResponses(),
                              );
                            },
                            fullscreenDialog: true,
                          )).then((value) {
                        if (value != null) {
                          details.addLogResponse(
                            action: value["action"],
                            comment: value["comment"],
                          );
                        }
                      });
                    }),
                onTap: () {
                  var items = details?.tracker ?? [];
                  List<Widget> _children = [];
                  _children = items
                      .map((i) => ListTile(
                            title: Text(i.type),
                            subtitle: Text(i.description),
                            trailing: JsonButton(json: i?.json),
                          ))
                      .toList();
                  showList(context, _children);
                },
              ),
              ListTile(
                title: Text("Tasks"),
                subtitle: Text("Tap To View"),
                trailing: IconButton(
                    icon: Icon(Icons.event),
                    onPressed: () async {
                      var _info = await details.client.getInfo();
                      Navigator.push<Map<String, dynamic>>(
                          context,
                          MaterialPageRoute(
                            builder: (BuildContext context) {
                              return new AddFollowUpExample(
                                userNumbers: _info.numbers,
                                prospectNumbers: details.numbers,
                              );
                            },
                            fullscreenDialog: true,
                          )).then((value) {
                        if (value != null) {
                          details.addFollowUp(
                            type: value["type"] ?? 0,
                            title: value["title"] ?? "",
                            description: value["description"] ?? "",
                            date: value["date_time"] ?? DateTime.now(),
                            dismissOnChange: value["dismiss_on_change"] ?? true,
                            alertType: value["alert_type"] ?? "",
                            alertOffset: value["alert_offset"] ?? 0,
                            prospectNumber: value["prospect_number"],
                            userNumber: value["user_number"],
                          );
                        }
                      });
                    }),
                onTap: () {
                  var items = details?.tasks ?? [];
                  List<Widget> _children = [];
                  _children = items
                      .map((i) => ListTile(
                            title: Text(i.type),
                            subtitle: Text(i?.dateTime?.toString() ?? ""),
                            trailing: JsonButton(json: i?.json),
                          ))
                      .toList();
                  showList(context, _children);
                },
              ),
            ],
          );
        },
      ),
    );
  }
}
