import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:get_it/get_it.dart';
import 'package:unify_api/unify.dart';

import '../../services/database/database.dart';
import '../../utils/utils.dart';
import '../scaffold/bloc.dart';
import 'bloc.dart';

class VideosBloc extends Bloc<VideosEvent, VideosState> {
  @override
  VideosState get initialState => InitialVideoState();

  @override
  Stream<VideosState> mapEventToState(
    VideosEvent event,
  ) async* {
    if (event is VideosCheck) {
      yield VideosLoadingState();
      //const _rows = kDefaultRows;
      final _item = await GetIt.instance<DatabaseService>()
          .videoDao
          .getLastVideo(event.user.id);
      if (_item == null) {
        yield* _checkStale(true, event.user);
      }
      yield VideosReadyState();
    }
    if (event is RefreshVideos) {
      yield VideosLoadingState();
      //const _rows = kDefaultRows;
      yield* _checkStale(false, event.user);
      event.scaffold.dispatch(ShowSnakBar('refreshed videos', event.context));
      yield VideosReadyState();
    }
  }
}

Stream<VideosState> _checkStale(bool checkStale, AccountInfo user) async* {
  if (checkStale) {
    final _item =
        await GetIt.instance<DatabaseService>().videoDao.getLastVideo(user.id);
    if (_item == null) {
      yield* _updateVideos(kDefaultRows, user);
    }
  } else {
    yield* _updateVideos(kDefaultRows, user);
  }
}

Stream<VideosState> _updateVideos(int _rows, AccountInfo user) async* {
  final _videos = await user.client.listVideosMyVideos(
    rows: _rows,
  );
  print('STILL Listing');
  if (_videos != null && _videos.isNotEmpty) {
    print('DONE LISTING');
    await GetIt.instance<DatabaseService>()
        .videoDao
        .batchUpdate(user.id, _videos);
  }
  yield VideosReadyState();
}
