import 'package:moor/moor.dart';
import 'package:unify_api/unify.dart' as api;


import '../database.dart';
import '../tables/group_item.dart';

part 'group_item.g.dart';

@UseDao(
  tables: [GroupItemRecords],
  queries: {
    'itemsByGroup':
      'SELECT * FROM groupItem WHERE group_id = :groupId AND user_id = :userId ORDER BY last_updated',
  }
)
class GroupItemDao extends DatabaseAccessor<DatabaseService>
    with _$GroupItemDaoMixin {
  GroupItemDao(DatabaseService db) : super(db);

  Future batchUpdateLead(String groupName, String groupId, String userId, List<api.Lead> leads) async{
    final _items = <GroupItemRecordsCompanion>[];
    final _time = DateTime.now();
    for(var lead in leads){
      final _item = GroupItemRecordsCompanion(
        groupName: addField(groupName),
        groupId: addField(groupId),
        userId: addField(userId),
        type: addField('lead'),
        itemId: addField(lead.id),
        lastUpdated: addField<DateTime>(_time),
      );
      _items.add(_item);
    }
    await into(groupItemRecords).insertAll(_items, orReplace: true);
    await cleanItems(userId, _time, 'lead');
  }

  Future batchUpdateContact(String groupName, String groupId, String userId, List<api.Contact> contacts) async{
    final _items = <GroupItemRecordsCompanion>[];
    final _time = DateTime.now();
    for(var contact in contacts){
      final _item = GroupItemRecordsCompanion(
        groupName: addField(groupName),
        groupId: addField(groupId),
        userId: addField(userId),
        type: addField('contact'),
        itemId: addField(contact.id),
        lastUpdated: addField<DateTime>(_time),
      );
      _items.add(_item);
    }
    await into(groupItemRecords).insertAll(_items, orReplace: true);
    await cleanItems(userId, _time, 'contact');
  }

  Future addItem(String groupName, String groupId, String userId, String itemId,
      String itemType) async {
    final _item = GroupItemRecordsCompanion(
        itemId: addField<String>(itemId),
        type: addField<String>(itemType),
        groupId: addField<String>(groupId),
        groupName: addField<String>(groupName),
        userId: addField<String>(userId));
    return await transaction(() async {
      await into(groupItemRecords).insert(_item, orReplace: true);
    });
  }

  Future cleanItems(String userId, DateTime lastUpdated, String type) async {
    return await (delete(groupItemRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) => l.lastUpdated.isBiggerThanValue(lastUpdated))
          ..where((l) => l.type.equals(type)))
        .go();
  }

  Future removeItem(String itemId, String userId, String groupId) async {
    return await transaction(() async {
      return (delete(groupItemRecords)
            ..where((l) => l.userId.equals(userId))
            ..where((l) => l.itemId.equals(itemId))
            ..where((l) => l.groupId.equals(groupId)))
          .go();
    });
  }
}
