import 'package:device_calendar/device_calendar.dart';

import 'classes/index.dart';

class CalendarUtils {
  CalendarUtils._();
  static final _deviceCalendarPlugin = DeviceCalendarPlugin();
  static Future<List<CalEvent>> getEventsForCalendar(
    String id, {
    DateTime startDate,
    DateTime endDate,
  }) async {
    final _events = await _deviceCalendarPlugin.retrieveEvents(
      id,
      RetrieveEventsParams(startDate: startDate, endDate: endDate),
    );
    if (_events?.data != null) {
      return _events.data
          .map((c) => CalEvent(
                id: c.eventId,
                title: c.title,
                start: c.start,
              ))
          .toList();
    }
    return [];
  }

  static Future<List<DeviceCal>> getCalendars() async {
    final _cals = await _deviceCalendarPlugin.retrieveCalendars();
    if (_cals?.data != null) {
      return _cals.data.map((c) => DeviceCal(id: c.id, name: c.name)).toList();
    }
    return [];
  }

  static void requestPermissions() async {
    await _deviceCalendarPlugin.requestPermissions();
  }

  static Future<bool> hasPermissions() async {
    final _result = await _deviceCalendarPlugin.hasPermissions();
    if (_result?.data != null && _result.data) {
      return _result.data;
    }
    return false;
  }
}
