import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_slidable/flutter_slidable.dart';
import 'package:get_it/get_it.dart';
import 'package:unify_api/tool.dart' as api;

import '../../../data/blocs/bloc.dart';
import '../../../data/classes/enum_list.dart';
import '../../../data/services/database/database.dart';
import '../../../plugins/local_storage/local_storage.dart';
import '../../contacts/common/more_button.dart';
import 'alerts_get_icon.dart';
import 'detailed_alert_tile.dart';

class AlertRowTile extends StatefulWidget {
  const AlertRowTile(
      {Key key,
      @required Dashboard alert,
      @required type,
      this.border,
      this.iconColor,
      this.onTap,
      this.onLongPress})
      : _alert = alert,
        _type = type,
        super(key: key);

  final Dashboard _alert;
  final ViewType _type;
  final BoxDecoration border;
  final Color iconColor;
  final VoidCallback onTap, onLongPress;

  @override
  _AlertRowTileState createState() => _AlertRowTileState();
}

class _AlertRowTileState extends State<AlertRowTile> {
  LocalStor storage;

  @override
  void initState() {
    storage = LocalStor('search')..init();

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final user = AuthBloc.currentUser(context);
    final info = AuthBloc.currentInfo(context);
    final _client =
        api.UnifyClient(user.username, user.password, domain: user.domain);
    final searchBloc = BlocProvider.of<SearchBloc>(context);

    switch (widget._type) {
      case ViewType.detailed:
        return Slidable(
          actionPane: SlidableDrawerActionPane(),
          actionExtentRatio: 0.25,
          child: DetailedAlertTile(alert: widget._alert),
          actions: <Widget>[
            IconSlideAction(
              caption: 'Delete',
              color: Colors.red,
              icon: Icons.delete,
              onTap: () {
                GetIt.instance<DatabaseService>()
                    .dashboardDao
                    .deleteAlert(user.userID, widget._alert.alertId);
                searchBloc.dispatch(TextChanged(text: '', user: info));
                _client.deleteDashboardAlert(widget._alert.alertId);
              },
            )
          ],
        );
        break;
      case ViewType.dense:
        return Slidable(
          actionPane: SlidableDrawerActionPane(),
          actionExtentRatio: 0.25,
          child: Container(
            decoration: widget.border,
            child: ListTile(
              onLongPress: () {},
              onTap: () {},
              title:
                  Text('${widget._alert.firstName} ${widget._alert.lastName}'),
              subtitle: Text(
                  '${_getTitle(widget._alert.folder)} - Priority: ${widget._alert.alertPriority}'),
              leading: getAlertIcon(widget._alert.alertType,
                  useColor: widget.iconColor, iconSize: 34),
              trailing: MoreButton(
                cell: widget._alert.cellPhone,
                office: widget._alert.officePhone,
                home: widget._alert.homePhone,
                email: widget._alert.email,
              ),
            ),
          ),
          secondaryActions: <Widget>[
            IconSlideAction(
              caption: 'Favorite',
              color: Colors.green,
              icon: Icons.star,
              onTap: () {},
            )
          ],
        );
        break;
    }
    return Container();
  }

  String _getTitle(String folder) {
    switch (folder) {
      case 'opportunity':
        return 'Opportunities';
      case 'customer':
        return 'Customer';
      case 'hotProspect':
        return 'Hot Prospect';
      case 'partners':
        return 'Partners';
      case 'pipeline':
        return 'My Pipeline';
      default:
    }
  }
}
