import 'package:after_layout/after_layout.dart';
import 'package:example_db/data/blocs/auth/auth_bloc.dart';
import 'package:flutter/material.dart';
import 'package:table_calendar/table_calendar.dart';
import 'package:unify_api/unify.dart';

import '../../generated/i18n.dart';
import '../../plugins/calendar/classes/calender.dart';
import 'common/index.dart';

class CalendarScreen extends StatefulWidget {
  @override
  _CalendarScreenState createState() => _CalendarScreenState();
}

class _CalendarScreenState extends State<CalendarScreen>
    with AfterLayoutMixin<CalendarScreen> {
  final CalendarController _calendarController = CalendarController();
  UnifyClient _client;
  DateTime _end = DateTime.now().add(Duration(days: 30));
  List _selectedEvents = [];
  DateTime _start = DateTime.now().subtract(Duration(days: 30));

  @override
  void afterFirstLayout(BuildContext context) {
    _calendarController.setSelectedDay(DateTime.now());
  }

  @override
  void dispose() {
    _calendarController.dispose();
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final _client = AuthBloc.currentInfo(context).client;
    return Scaffold(
      // appBar: AppBar(
      //   leading: MenuButton(),
      //   title: Text('Calendar'),
      // ),
      body: LayoutBuilder(
        builder: (context, dimens) {
          if (dimens.maxWidth > 600) {
            return Row(
              children: <Widget>[
                Container(
                  width: 400,
                  child: _buildCal(_client),
                ),
                Expanded(
                  child: Container(child: _buildEvents()),
                ),
              ],
            );
          }
          return Column(
            mainAxisSize: MainAxisSize.max,
            children: <Widget>[
              _buildCal(_client),
              Expanded(child: _buildEvents()),
            ],
          );
        },
      ),
    );
  }

  Widget _buildEvents() {
    return Column(
      children: <Widget>[
        const SizedBox(height: 8.0),
        if (_selectedEvents != null) ...[
          if (_selectedEvents.isEmpty) ...[
            Expanded(
              child: Center(
                child: Text('No Events Found'),
              ),
            )
          ] else ...[
            Expanded(
                child: ListView(
              children: _selectedEvents
                  .map((event) => Container(
                        decoration: BoxDecoration(
                          border: Border.all(width: 0.8),
                          borderRadius: BorderRadius.circular(12.0),
                        ),
                        margin: const EdgeInsets.symmetric(
                            horizontal: 8.0, vertical: 4.0),
                        child: ListTile(
                          title: Text(event.toString()),
                          onTap: () => print('$event tapped!'),
                        ),
                      ))
                  .toList(),
            )),
          ],
        ] else ...[
          Expanded(
            child: Center(
              child: CircularProgressIndicator(),
            ),
          )
        ],
      ],
    );
  }

  Widget _buildCal(UnifyClient client) {
    return Column(
      children: <Widget>[
        UnifyCalendar(
          client: client,
          calendarController: _calendarController,
          start: _start,
          end: _end,
          onDaySelected: (day, events) {
            if (mounted) {
              setState(() {
                _selectedEvents = events;
              });
            }
          },
          onVisibleDaysChanged: (first, last, format) {
            print('CALLBACK: _onVisibleDaysChanged => $format');
          },
        ),
        const SizedBox(height: 8.0),
        Row(
          mainAxisAlignment: MainAxisAlignment.spaceAround,
          children: <Widget>[
            ActionChip(
              avatar: Icon(Icons.share),
              label: Text(I18n.of(context).calendarShare),
              onPressed: () {},
            ),
            ActionChip(
              avatar: Icon(Icons.today),
              label: Text(I18n.of(context).calendarToday),
              onPressed: () {
                _calendarController.setSelectedDay(DateTime.now(),
                    runCallback: true);
              },
            ),
            ActionChip(
              avatar: Icon(Icons.refresh),
              label: Text(I18n.of(context).calendarRefresh),
              onPressed: () {
                _calendarController.setSelectedDay(DateTime.now(),
                    runCallback: true);
              },
            ),
          ],
        ),
      ],
    );
  }
}
