import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:unify_api/tool.dart';

import '../../data/utils/is_empty.dart';

class ContactsListView extends StatelessWidget {
  ContactsListView({@required this.contacts});
  final List<LoanContact> contacts;

  final money = NumberFormat.simpleCurrency();

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: Text('Loan Contacts')),
      body: ListView.builder(
          itemCount: contacts.length,
          itemBuilder: (context, index) {
            return ExpansionTile(
              leading: Icon(Icons.person),
              title: Text(contacts[index].type),
              children: <Widget>[
                ListTile(
                  leading: Icon(Icons.person),
                  title: Text('Contact Name'),
                  subtitle: Text(isEmpty(
                      string: contacts[index].name,
                      returnStr: 'No Name Found')),
                ),
                if (contacts[index].cellPhone != null)
                  ListTile(
                    leading: Icon(Icons.phone),
                    title: Text('Cell Phone'),
                    subtitle: Text(isEmpty(
                        string: contacts[index].cellPhone,
                        returnStr: 'No Number Found')),
                    //TODO
                    //trailing: ,
                  ),
                if (contacts[index].officePhone != null)
                  ListTile(
                    leading: Icon(Icons.work),
                    title: Text('Office Phone'),
                    subtitle: Text(isEmpty(
                        string: contacts[index].officePhone,
                        returnStr: 'No Number Found')),
                    //TODO
                    //trailing:
                  ),
                if (contacts[index].address != null)
                  ListTile(
                    leading: Icon(Icons.map),
                    title: Text('Current Address'),
                    subtitle: Text(formatAddress(loan: contacts[index])),
                  ),
                if (contacts[index].email != null)
                  ListTile(
                      leading: Icon(Icons.email),
                      title: Text('Email'),
                      subtitle: Text(isEmpty(
                          string: contacts[index].email,
                          returnStr: 'No Email Found'))),
                if (contacts[index].type == 'Loan Officer')
                  ListTile(
                      leading: Icon(Icons.info),
                      title: Text('NMLS#'),
                      subtitle: Text(isEmpty(
                          string: contacts[index].nmls, returnStr: 'ID')))
              ],
            );
          }),
    );
  }

  String formatAddress({LoanContact loan}) {
    final address = loan.address;

    if (address == null || address.raw().isEmpty) {
      return 'No Address Found';
    }
    return '${address.state} ${address.city}\n${address.state} ${address.zip}';
  }
}
