import 'package:flip_card/flip_card.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:get_it/get_it.dart';
import 'package:numberpicker/numberpicker.dart';

import '../../../data/blocs/auth/auth_bloc.dart';
import '../../../data/classes/enum_list.dart';
import '../../../data/services/database/database.dart';
import '../../../data/utils/utils.dart';
import '../../alerts/common/tile.dart';
import '../../common/raised_card.dart';

class DashboardAlertsCard extends StatefulWidget {
  static const String cardKey = 'card_alerts';
  final String tag;
  final int id;
  final bool preview;

  const DashboardAlertsCard(
      {Key key, this.tag, @required this.id, this.preview = false})
      : super(key: key);

  @override
  _DashboardAlertsCardState createState() => _DashboardAlertsCardState();
}

class _DashboardAlertsCardState extends State<DashboardAlertsCard> {
  int _rowsPerPage = 3;
  final _dropItems = ['Opportunity', 'Customer', 'Hotprospects', 'Pipline'];
  final _folders = [
    DashboardAlertType.opportunity,
    DashboardAlertType.customer,
    DashboardAlertType.hotProspect,
    DashboardAlertType.pipeline,
  ];
  String actualDropdown;
  final _pageController = PageController();

  GlobalKey<FlipCardState> cardKey = GlobalKey<FlipCardState>();

  @override
  Widget build(BuildContext context) {
    final _user = AuthBloc.currentUser(context);
    return FlipCard(
      flipOnTouch: false,
      speed: 450,
      key: cardKey,
      front: RaisedChild(Padding(
        padding: EdgeInsets.all(20),
        child: LayoutBuilder(
            builder: (context, dimens) => Container(
                  height: (_rowsPerPage + 1) * 110.0,
                  width: dimens.maxWidth > kBreakpoint
                      ? dimens.maxWidth * 0.2
                      : dimens.maxWidth,
                  child: Column(
                    mainAxisSize: MainAxisSize.min,
                    mainAxisAlignment: MainAxisAlignment.spaceAround,
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: <Widget>[
                      Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          Expanded(
                              child: Column(
                            mainAxisAlignment: MainAxisAlignment.start,
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: <Widget>[
                              Text(
                                "Dashboard Alerts",
                                maxLines: 1,
                                style: TextStyle(color: Colors.orange),
                                textScaleFactor: 1,
                              ),
                              Text(
                                actualDropdown ?? _dropItems.first,
                                textScaleFactor: 1,
                                style: TextStyle(
                                  fontWeight: FontWeight.w700,
                                  fontSize:
                                      MediaQuery.of(context).size.width < 350
                                          ? 20
                                          : 24,
                                ),
                              ),
                            ],
                          )),
                          Container(
                              child: DropdownButton(
                            isDense: true,
                            value: actualDropdown ?? _dropItems?.first,
                            onChanged: (value) => setState(() {
                              actualDropdown = value;
                            }),
                            items: _dropItems.map((String title) {
                              return DropdownMenuItem(
                                value: title,
                                child: Text(
                                  title,
                                  style: TextStyle(
                                    color: Colors.blue,
                                    fontWeight: FontWeight.w400,
                                    fontSize: 14,
                                  ),
                                ),
                              );
                            }).toList(),
                          )),
                        ],
                      ),
                      Padding(
                        padding: EdgeInsets.only(bottom: 4),
                      ),
                      _buildStreamBuilder(_user, actualDropdown),
                      Container(
                        //height: 75,
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            IconButton(
                              icon: Icon(Icons.navigate_before, size: 35),
                              onPressed: () {
                                _pageController.previousPage(
                                    curve: Curves.easeIn,
                                    duration:
                                        const Duration(milliseconds: 200));
                              },
                            ),
                            IconButton(
                              icon: Icon(Icons.settings),
                              onPressed: () {
                                if (!widget.preview)
                                  cardKey.currentState.toggleCard();
                              },
                            ),
                            IconButton(
                              icon: Icon(Icons.navigate_next, size: 35),
                              onPressed: () {
                                _pageController.nextPage(
                                    curve: Curves.easeIn,
                                    duration:
                                        const Duration(milliseconds: 200));
                              },
                            ),
                          ],
                        ),
                      )
                    ],
                  ),
                )),
      )),
      back: RaisedChild(
        Padding(
          padding: EdgeInsets.all(24),
          child: LayoutBuilder(
            builder: (context, dimens) => Container(
              height: (_rowsPerPage + 1) * 110.0,
              width: dimens.maxWidth > kBreakpoint
                  ? dimens.maxWidth * 0.2
                  : dimens.maxWidth,
              child: Column(
                mainAxisAlignment: MainAxisAlignment.start,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: <Widget>[
                      IconButton(
                        icon: Icon(Icons.navigate_before, size: 35),
                        onPressed: () => cardKey.currentState.toggleCard(),
                      ),
                      IconButton(
                        icon: Icon(Icons.delete),
                        onPressed: () => deleteWidget(_user, widget.id),
                      )
                    ],
                  ),
                  SizedBox(height: 25),
                  Container(
                    child: Column(
                      children: <Widget>[
                        Divider(),
                        ListTile(
                            title: Text('Alerts Per Row: '),
                            trailing: FlatButton(
                                child: Text('$_rowsPerPage',
                                    style: Theme.of(context).textTheme.title),
                                onPressed: () {
                                  _showDialog();
                                })),
                        Divider(),
                      ],
                    ),
                  ),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }

  void _showDialog() {
    showDialog<int>(
        context: context,
        builder: (BuildContext context) {
          return new NumberPickerDialog.integer(
            minValue: 1,
            maxValue: 10,
            title: new Text("Number of Alerts"),
            initialIntegerValue: _rowsPerPage,
          );
        }).then((int value) {
      if (value != null) {
        setState(() => _rowsPerPage = value);
      }
    });
  }

  void deleteWidget(User user, int id) async {
    await GetIt.instance<DatabaseService>().homeDao.removeItem(id, user.userID);
  }

  StreamBuilder<List<Dashboard>> _buildStreamBuilder(
      User _user, String actualDropdown) {
    actualDropdown ??= _dropItems.first;
    return StreamBuilder<List<Dashboard>>(
        stream: GetIt.instance<DatabaseService>()
            .dashboardDao
            .watchAlertsByFolder(
                _user.userID, describeEnum(_getType(actualDropdown))),
        builder: (context, snapshot) {
          if (snapshot.hasData) {
            final _alerts = snapshot.data;
            return _DashboardListCard(
                alerts: _alerts,
                rowsPerPage: _rowsPerPage,
                pageController: _pageController);
          } else {
            return Center(child: CircularProgressIndicator());
          }
        });
  }

  DashboardAlertType _getType(String title) {
    switch (title) {
      case 'Opportunity':
        return DashboardAlertType.opportunity;
      case 'Customer':
        return DashboardAlertType.customer;
      case 'Hotprospects':
        return DashboardAlertType.hotProspect;
      case 'Pipline':
        return DashboardAlertType.pipeline;
      default:
    }
  }
}

class _DashboardListCard extends StatelessWidget {
  final int rowsPerPage;
  final List alerts;
  final DashboardAlertType type;
  final PageController pageController;

  _DashboardListCard(
      {this.pageController,
      this.alerts,
      this.type = DashboardAlertType.opportunity,
      this.rowsPerPage = 3});

  @override
  Widget build(BuildContext context) {
    if (alerts == null) {
      return Container(child: Center(child: CircularProgressIndicator()));
    }

    if (alerts.isEmpty) {
      return Container(
        child: Expanded(
          child: Center(
            child: Text(
              'No Alerts for Today',
              textAlign: TextAlign.center,
              style: TextStyle(
                fontWeight: FontWeight.w600,
                fontSize: 34,
              ),
            ),
          ),
        ),
      );
    }

    final _total = alerts?.length ?? 0;
    final _pages = (_total / rowsPerPage).ceil();
    return Expanded(
      child: PageView.builder(
          controller: pageController,
          itemCount: _pages,
          itemBuilder: (context, index) {
            List<Dashboard> _items =
                alerts.skip(index * rowsPerPage).take(rowsPerPage).toList();
            return Column(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: <Widget>[
                // Padding(padding: EdgeInsets.all(5)),
                for (var item in _items) ...[
                  AlertRowTile(
                    alert: item,
                    type: ViewType.detailed,
                  )
                ]
              ],
            );
          }),
    );
  }
}
