import 'dart:convert';

import 'package:example_db/data/blocs/auth/bloc.dart';
import 'package:example_db/data/services/database/database.dart';
import 'package:flip_card/flip_card.dart';
import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:get_it/get_it.dart';
import 'package:hive/hive.dart';
import 'package:http/http.dart' as http;
import 'package:intl/intl.dart';

import '../../../plugins/local_storage/local_storage.dart';
import '../../../plugins/location/location.dart';
import '../../common/index.dart';

class CurrentWeatherCard extends StatefulWidget {
  static const String cardKey = 'card_weather';
  static const BoxConstraints constraints = BoxConstraints(
    minHeight: 50.0,
    minWidth: 50.0,
    maxHeight: double.infinity,
    maxWidth: 650.0,
  );
  final int id;
  final bool preview;

  const CurrentWeatherCard({Key key, this.id, this.preview = false}) : super(key: key);

  @override
  _CurrentWeatherCardState createState() => _CurrentWeatherCardState();
}

class _CurrentWeatherCardState extends State<CurrentWeatherCard> {
  Box box;
  LocalStor storage;
  String error;
  String _icon = 'none';
  String _name = '';
  double _temp = 70.0;
  GlobalKey<FlipCardState> cardKey = GlobalKey<FlipCardState>();

  @override
  void initState() {
    init();
    storage = LocalStor('current_weather_${widget?.id ?? 0}');
    try {
      loadWeather();
    } catch (e) {}
    super.initState();
  }

  Future init() async {
    return box = await Hive.openBox('current_weather_${widget?.id ?? 0}');
  }

  @override
  Widget build(BuildContext context) {
    final _user = AuthBloc.currentUser(context);
    if (_temp == null) {
      return InkWell(
        child: Center(child: CircularProgressIndicator()),
        onTap: loadWeather,
      );
    }
    return FutureBuilder(
        future: init(),
        builder: (context, snapshot) {
          return GestureDetector(
            onDoubleTap: () => {
              if (!widget.preview)
                cardKey.currentState.toggleCard()
            },
            child: FlipCard(
              flipOnTouch: false,
              speed: 450,
              key: cardKey,
              front: RaisedChild(
                Padding(
                  padding: EdgeInsets.all(24),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: <Widget>[
                      Builder(
                        builder: (_) {
                          if(_name == null || _temp == null|| _icon == null ){
                            return CircularProgressIndicator();
                          }
                          return Column(
                            mainAxisAlignment: MainAxisAlignment.center,
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: <Widget>[
                              Text(
                                _name ?? '',
                                textScaleFactor: 1,
                                style: TextStyle(color: Colors.blueAccent),
                              ),
                              Text(
                                '$_temp°F',
                                textScaleFactor: 1,
                                maxLines: 1,
                                style: TextStyle(
                                    fontWeight: FontWeight.w700, fontSize: 24),
                              ),
                              Text(
                                _icon ?? '',
                                textScaleFactor: 1,
                                maxLines: 1,
                                style: TextStyle(
                                  fontWeight: FontWeight.w200,
                                  fontSize: 16,
                                ),
                              ),
                            ],
                          );
                        }
                      ),
                      Material(
                        color: _getIconColor(_temp ?? ''),
                        borderRadius: BorderRadius.circular(24),
                        child: Center(
                          child: Padding(
                            padding: EdgeInsets.all(16),
                            child: Icon(
                              _getWeatherIcon(_icon ?? ''),
                              size: 30,
                              color: Colors.white,
                            ),
                          ),
                        ),
                      )
                    ],
                  ),
                ),
              ),
              back: RaisedChild(Container(
                height: 110,
                child: Center(
                    child: FlatButton.icon(
                  label: Text('Delete Widget'),
                  icon: Icon(Icons.delete),
                  onPressed: () {
                    deleteWidget(_user, widget.id);
                  },
                )),
              )),
            ),
          );
        });
  }

  void deleteWidget(User user, int id) async {
    await GetIt.instance<DatabaseService>().homeDao.removeItem(id, user.userID);
  }

  IconData _getWeatherIcon(String name) {
    var _name = name.toLowerCase().trim();
    if (_name.contains("rain")) return FontAwesomeIcons.cloudRain;
    if (_name.contains("haze")) return Icons.dehaze;
    if (_name.contains("smog")) return FontAwesomeIcons.smog;
    if (_name.contains("snow")) return FontAwesomeIcons.snowflake;
    if (_name.contains("high")) return FontAwesomeIcons.temperatureHigh;
    if (_name.contains("low")) return FontAwesomeIcons.temperatureLow;
    if (_name.contains("cloud")) return FontAwesomeIcons.cloudSun;
    if (_name.contains("showers")) return FontAwesomeIcons.cloudShowersHeavy;
    if (_name.contains("storm")) return FontAwesomeIcons.pooStorm;
    return FontAwesomeIcons.cloudSun;
  }

  Color _getIconColor(double value) {
    if (value >= 90.0) return Colors.redAccent;
    if (value >= 70) return Colors.amber;
    if (value >= 50.0) return Colors.lightBlue;
    return Colors.blue;
  }

  void loadWeather() async {
    if (mounted) setState(() {});
    final location = await LocationUtils.getLocation();
    if (location != null) {
      final lon = location.longitude;
      final lat = location.latitude;
      try {
        final weatherResponse = await http.get(
            'https://api.openweathermap.org/data/2.5/weather?APPID=0721392c0ba0af8c410aa9394defa29e&lat=${lat.toString()}&lon=${lon.toString()}');
        if (weatherResponse.statusCode == 200) {
          final weatherData =
              new CurrentWeather.fromJson(jsonDecode(weatherResponse.body));
          if (mounted)
            setState(() async {
              _icon = weatherData.name;
              _temp = weatherData.temp;
              _name = DateFormat.yMd().format(weatherData.date);
            });
          box.put('weatherName', weatherData.name);
          box.put('weatherDate', weatherData.date.toString());
          box.put('weatherTemp', weatherData.temp);
          box.put('weatherMain', weatherData.main);
          box.put('weatherIcon', weatherData.icon);
        }
      } catch (e) {
        if (box.isOpen && box.isNotEmpty) {
          if (mounted) {
            setState(() {
              _temp =
                  double.tryParse(box.get('weatherTemp').toString()) ?? 70.0;
              _icon = box.get('weatherName');
              final _date = box.get('weatherDate');
              if (_date != null)
                _name = DateFormat.yMd().format(DateTime.tryParse(_date));
            });
          }
        }
      }
      setState(() {});
    }
  }

  bool _isDisposed = false;
  @override
  void dispose() {
    _isDisposed = true;
    super.dispose();
  }
}

double kelvinToFarienhight(double value) {
  var _result = (value - 273.15) * 9 / 5 + 32;
  return _result.roundToDouble();
}

class CurrentWeather {
  final DateTime date;
  final String name;
  final double temp;
  final String main;
  final String icon;

  CurrentWeather({this.date, this.name, this.temp, this.main, this.icon});

  factory CurrentWeather.fromJson(Map<String, dynamic> json) {
    return CurrentWeather(
      date: new DateTime.fromMillisecondsSinceEpoch(json['dt'] * 1000,
          isUtc: false),
      name: json['name'],
      temp: kelvinToFarienhight(json['main']['temp'].toDouble()),
      main: json['weather'][0]['main'],
      icon: json['weather'][0]['icon'],
    );
  }
}
