import 'package:example_db/ui/dashboard/cards/action_button.dart';
import 'package:flip_card/flip_card.dart';
import 'package:flutter/material.dart';
import 'package:get_it/get_it.dart';

import '../../../data/blocs/bloc.dart';
import '../../../data/services/database/database.dart';
import '../../common/index.dart';

class VideoViewsTotal extends StatelessWidget {
  static const String cardKey = 'card_video';
  final int id;
  final bool preview; 

  const VideoViewsTotal({Key key, @required this.id, this.preview = false}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    GlobalKey<FlipCardState> flipKey = GlobalKey<FlipCardState>();
    final user = AuthBloc.currentUser(context);
    return GestureDetector(
      onDoubleTap: () => {
        if(!preview)
          flipKey.currentState.toggleCard()
      },
      child: FlipCard(
        flipOnTouch: false,
        key: flipKey,
        front: RaisedChild(
          LayoutBuilder(
            builder: (context, constraints) => Container(
              height: 125,
              width: constraints.maxWidth > 850
                  ? constraints.maxWidth / 2
                  : constraints.maxWidth,
              child: Padding(
                padding: const EdgeInsets.all(24.0),
                child: Stack(
                  children: <Widget>[
                    Opacity(
                      opacity: user.videoEnabled ? 1 : 0.5,
                      child: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          crossAxisAlignment: CrossAxisAlignment.center,
                          children: <Widget>[
                            Column(
                              mainAxisAlignment: MainAxisAlignment.center,
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: <Widget>[
                                Text(
                                  'Unify Video Total Views',
                                  style: TextStyle(
                                    color: Colors.blueAccent,
                                    fontSize: 125 * 0.12,
                                  ),
                                ),
                                FutureBuilder(
                                    future: totalViews(user),
                                    builder: (context, snapshot) {
                                      if (snapshot.hasData &&
                                          snapshot.data != null) {
                                        return Text(
                                          '${snapshot.data.toString()}',
                                          style: TextStyle(
                                            fontWeight: FontWeight.w700,
                                            fontSize: 125 * 0.3,
                                          ),
                                        );
                                      } else {
                                        return CircularProgressIndicator();
                                      }
                                    }),
                              ],
                            ),
                            Material(
                                color: Colors.blue,
                                borderRadius: BorderRadius.circular(24.0),
                                child: Center(
                                    child: Padding(
                                  padding: const EdgeInsets.all(16.0),
                                  child: Icon(
                                    Icons.timeline,
                                    color: Colors.white,
                                    size: 125 * 0.15,
                                  ),
                                )))
                          ]),
                    ),
                    if (!user.videoEnabled) ...[
                      Center(
                        child: Text(
                          'Video Not Enabled',
                          style: TextStyle(
                            fontWeight: FontWeight.w600,
                            fontSize: 25,
                          ),
                        ),
                      ),
                    ]
                  ],
                ),
              ),
            ),
          ),
        ),
        back: RaisedChild(LayoutBuilder(
          builder: (context, constraints) => Container(
            height: 125,
            width: constraints.maxWidth > 850
                ? constraints.maxWidth * 0.2
                : constraints.maxWidth,
            child: Center(
              child: FlatButton.icon(
                label: Text('Delete Widget'),
                icon: Icon(Icons.delete),
                onPressed: () => deleteWidget(user, id),
              )
            ),
          ),
        )),
      ),
    );
  }

  void deleteWidget(User user, int id) async {
    await GetIt.instance<DatabaseService>().homeDao.removeItem(id, user.userID);
  }

  Future<int> totalViews(User user) async {
    return await GetIt.instance<DatabaseService>()
        .videoDao
        .getTotalViews(user.userID);
  }
}
