import 'package:flutter/material.dart';
import 'package:flutter_material_color_picker/flutter_material_color_picker.dart';
import 'package:hive/hive.dart';

import '../../common/raised_card.dart';
import '../cards/action_button.dart';

class BuildActionScreen extends StatefulWidget {
  final int id;

  const BuildActionScreen({
    @required this.id,
    Key key,
  }) : super(key: key);

  @override
  _BuildActionScreenState createState() => _BuildActionScreenState();
}

class _BuildActionScreenState extends State<BuildActionScreen> {
  Box box;
  int colorSelect;
  String title, subtitle;
  IconData icon;
  VoidCallback onTap;
  bool load;
  bool changed = false;
  final _dropItems = ['Add Contact', 'Add Lead', 'calculator'];
  // final _dropItems = [DropdownMenuItem(value: 'Manage Contacts', child: Text('Manage Contacts')),
  //   DropdownMenuItem(value: 'Manage Leads', child: Text('Manage Leads')),
  //   DropdownMenuItem(value: 'Manage Loans', child: Text('Manage Loans')),
  //   DropdownMenuItem(value: 'Contact Groups', child: Text('Contact Groups')),
  //   DropdownMenuItem(value: 'Lead Groups', child: Text('Lead Groups'))
  //   ];
  String selected;

  @override
  void initState() {
    box = Hive.box('action_storage_${widget.id ?? 0}');

    print(widget.id);
    load = false;
    loadValues();
    super.initState();
  }

  void loadValues() {
    colorSelect = box.get('action_color') ?? Colors.teal.value;
    title = box.get('action_title') ?? 'Add';
    subtitle = box.get('action_subtitle') ?? 'Create New Action';
    icon = IconData(box.get('action_icon'), fontFamily: 'MaterialIcons') ??
        Icons.add;
  }

  @override
  Widget build(BuildContext context) {
    if (box.isOpen) {
      return Scaffold(
        resizeToAvoidBottomInset: false,
        appBar: AppBar(
          title: Text('Edit Action'),
          actions: <Widget>[
            IconButton(
              icon: Icon(Icons.check),
              onPressed: _saveButton,
            )
          ],
        ),
        body: SingleChildScrollView(
          child: Form(
            onChanged: () => changed = true,
            onWillPop: () => _prompt(context, 'Warning - Unsaved Changes',
                'Do you want to discard changes?'),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                Container(
                  height: 250,
                  width: MediaQuery.of(context).size.width,
                  decoration: BoxDecoration(
                      border: Border(
                          bottom: BorderSide(
                              color: Theme.of(context).dividerColor)),
                      color: Colors.grey[200]),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: <Widget>[
                      Center(
                        child: RaisedChild(
                          Padding(
                            padding: EdgeInsets.all(20),
                            child: ConstrainedBox(
                              constraints: ActionButtonCard.constraints,
                              child: ActionButtonUI(
                                box: box,
                                title: title,
                                subtitle: subtitle,
                                color: colorSelect,
                                icon: icon.codePoint,
                                fontFam: icon.fontFamily,
                                fontPack: icon.fontPackage,
                                load: false,
                              ),
                            ),
                          ),
                        ),
                      )
                    ],
                  ),
                ),
                Container(
                  padding: EdgeInsets.all(10),
                  child: Column(
                    children: <Widget>[
                      TextFormField(
                        decoration: InputDecoration(
                            hintText: 'Action Title',
                            labelText: box.get('action_title')),
                        onChanged: (val) {
                          setState(() {
                            title = val;
                          });
                        },
                      ),
                      //Padding(padding: EdgeInsets.all(10)),
                      TextFormField(
                        decoration: InputDecoration(
                            hintText: 'Action Details',
                            labelText: box.get('action_subtitle')),
                        onChanged: (val) {
                          setState(() {
                            subtitle = val;
                          });
                        },
                      ),
                    ],
                  ),
                ),
                Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Row(
                    children: <Widget>[
                      Expanded(
                        child: Text('Select Color',
                            style: Theme.of(context).textTheme.title),
                      ),
                      InkWell(
                        onTap: _openColorPicker,
                        child: CircleAvatar(
                          backgroundColor: Color(colorSelect) ?? Colors.teal,
                        ),
                      ),
                    ],
                  ),
                ),
                Divider(),
                Padding(
                  padding: const EdgeInsets.all(8),
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.spaceAround,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      Text('Select an Icon',
                          style: Theme.of(context).textTheme.title,
                          textAlign: TextAlign.start),
                      Row(
                        crossAxisAlignment: CrossAxisAlignment.center,
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: <Widget>[
                          IconButton(
                            icon: Icon(Icons.add),
                            iconSize: 34,
                            onPressed: () {
                              setState(() {
                                icon = Icons.add;
                                changed = true;
                              });
                            },
                          ),
                          IconButton(
                            icon: Icon(Icons.edit),
                            iconSize: 34,
                            onPressed: () {
                              setState(() {
                                icon = Icons.edit;
                                changed = true;
                              });
                            },
                          ),
                          IconButton(
                            icon: Icon(Icons.calendar_today),
                            iconSize: 34,
                            onPressed: () {
                              setState(() {
                                icon = Icons.calendar_today;
                                changed = true;
                              });
                            },
                          ),
                          IconButton(
                            icon: Icon(Icons.people),
                            iconSize: 34,
                            onPressed: () {
                              setState(() {
                                icon = Icons.people;
                                changed = true;
                              });
                            },
                          ),
                          IconButton(
                            icon: Icon(Icons.favorite),
                            iconSize: 34,
                            onPressed: () {
                              setState(() {
                                icon = Icons.favorite;
                                changed = true;
                              });
                            },
                          ),
                          IconButton(
                            icon: Icon(Icons.videocam),
                            iconSize: 34,
                            onPressed: () {
                              setState(() {
                                icon = Icons.videocam;
                                changed = true;
                              });
                            },
                          ),
                          IconButton(
                            icon: Icon(Icons.folder),
                            iconSize: 34,
                            onPressed: () {
                              setState(() {
                                icon = Icons.folder;
                                changed = true;
                              });
                            },
                          )
                        ],
                      )
                    ],
                  ),
                ),
                Divider(),
                Padding(
                  padding: const EdgeInsets.all(10.0),
                  child: Row(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: <Widget>[
                      Expanded(
                        child: Text('Select an Action',
                            style: Theme.of(context).textTheme.title),
                      ),
                      // Container(
                      //   child: DropdownButton(
                      //     isDense: true,
                      //     value: selected ?? 'Choose Action',
                      //     onChanged: (value) => setState(() {
                      //       if(mounted){
                      //         setState(() {
                      //           selected = value;
                      //         });
                      //       }
                      //     }),
                      //     items: <String>['Contacts', 'Leads', 'Loans', 'Calculator'].map<DropdownMenuItem<String>>((String value) {
                      //       return DropdownMenuItem<String>(
                      //         value: value,
                      //         child: Text(value),
                      //       );
                      //     }).toList(),
                      //   ),
                      // )
                    ],
                  ),
                )
              ],
            ),
          ),
        ),
      );
    }
  }

  void _saveButton() {
    box.put('action_color', colorSelect);
    box.put('action_icon', icon.codePoint);
    box.put('action_title', title);
    box.put('action_subtitle', subtitle);
    setState(() {
      load = true;
      changed = false;
    });
    Navigator.of(context, rootNavigator: true).pop(true);
  }

  Future<T> _openDialog<T>(Widget content) {
    return showDialog<T>(
        context: context,
        builder: (_) {
          return AlertDialog(
            contentPadding: EdgeInsets.all(6),
            title: Text('Select a Color'),
            content: content,
          );
        });
  }

  void _openColorPicker() async {
    await _openDialog(MaterialColorPicker(
        colors: fullMaterialColors,
        allowShades: false,
        selectedColor: Color(colorSelect),
        onMainColorChange: (color) {
          setState(() {
            colorSelect = color.value;
            changed = true;
          });
        }));
  }

  Future<bool> _prompt(
      BuildContext context, String title, String detail) async {
    if (!changed) {
      return Future<bool>.value(true);
    } else {
      return await showConfirmationPopup(
        context,
        title: 'Warning - Unsaved Changes',
        detail: 'Do you want to discard all changes?',
      );
    }
  }

  Future<bool> showConfirmationPopup(BuildContext context,
      {String title = "Info", String detail = ""}) async {
    return await showDialog<bool>(
        context: context,
        builder: (context) => AlertDialog(
              title: Text(title),
              content: Text(detail),
              actions: <Widget>[
                RaisedButton(
                    child: Text('Confirm'),
                    // isDestructive: true,
                    onPressed: () {
                      Navigator.of(context, rootNavigator: true).pop(true);
                    }),
                RaisedButton(
                    child: Text('Cancel'),
                    // isDestructive: false,
                    onPressed: () {
                      Navigator.of(context, rootNavigator: true).pop(false);
                    }),
              ],
            ));
  }
}
