import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get_it/get_it.dart';
import 'package:unify_api/unify.dart' as api;

import '../../../data/blocs/bloc.dart';
import '../../../data/classes/enum_list.dart';
import '../../../data/services/database/dao/loan.dart';
import '../../../data/services/database/database.dart';
import 'tile.dart';

class LoanFolderScreen extends StatelessWidget {
  const LoanFolderScreen({
    Key key,
    @required this.folder,
    @required this.query,
    @required this.scope,
  }) : super(key: key);

  final api.LoansFolder folder;
  final String query;
  final LoanScope scope;

  @override
  Widget build(BuildContext context) {
    int favId = 0;
    final loans = BlocProvider.of<LoansBloc>(context);
    final _user = AuthBloc.currentUser(context);
    final db = GetIt.instance<DatabaseService>();
    return StreamBuilder<LoansState>(
        stream: loans.state,
        builder: (context, snapshot) {
          final _state = snapshot.data;
          return Column(
            children: <Widget>[
              if (_state is LoansLoadingState && _state.folder == folder)
                LinearProgressIndicator(),
              Expanded(
                child: Container(
                  child: StreamBuilder<List<Loan>>(
                      stream: _getList(db, _user, folder),
                      builder: (context, snapshot) {
                        if (snapshot.hasData) {
                          final _loans = snapshot.data;
                          if (_loans.isEmpty) {
                            if (_state is LoansLoadingState) {
                              return Column(
                                mainAxisAlignment: MainAxisAlignment.center,
                                crossAxisAlignment: CrossAxisAlignment.center,
                                children: [
                                  Text('No Loans Found'),
                                  SizedBox(height: 20.0),
                                  CircularProgressIndicator(),
                                ],
                              );
                            }
                            return Center(child: Text('No Loans Found'));
                          }
                          return Scrollbar(
                            child: RefreshIndicator(
                              child: ListView.separated(
                                separatorBuilder: (context, index) =>
                                    Divider(height: 1),
                                itemCount: _loans.length,
                                itemBuilder: (context, index) {
                                  final _loan = _loans[index];
                                  return Container(
                                    child: StreamBuilder<List<Favorite>>(
                                        stream: db.favoritesDao
                                            .watchFavoriteByTypeId(
                                                _user.userID, _loan.loanId),
                                        builder: (context, snapshot) {
                                          if (snapshot.hasData &&
                                              snapshot.data.isNotEmpty) {
                                            favId =
                                                snapshot.data.first.favoriteId;
                                          }
                                          return LoanRowTile(
                                            favId: favId,
                                            favorite: snapshot.hasData &&
                                                snapshot.data.isNotEmpty,
                                            loan: _loan,
                                            type: ViewType.detailed,
                                          );
                                        }),
                                  );
                                },
                              ),
                              onRefresh: () async {
                                refresh(context, folder);
                                await Future.delayed(Duration(seconds: 3));
                              },
                            ),
                          );
                        }
                        return Center(child: CircularProgressIndicator());
                      }),
                ),
              ),
            ],
          );
        });
  }

  Stream<List<Loan>> _getList(
      DatabaseService db, User _user, api.LoansFolder folder) {
    if (query == null) {
      return db.loansDao.watchLoansByFolder(_user.userID, describeEnum(folder));
    }
    return db.loansDao
        .searchLoans(
          _user.userID,
          query,
          filter: scope,
          folder: folder,
        )
        .asStream();
  }

  static refresh(BuildContext context, api.LoansFolder folder) {
    final loans = BlocProvider.of<LoansBloc>(context);
    final _user = AuthBloc.currentInfo(context);
    final scaffold = BlocProvider.of<ScaffoldBloc>(context);
    return loans.dispatch(RefreshLoans(
        info: _user, folder: folder, scaffold: scaffold, context: context));
  }
}
