import 'package:flutter/material.dart';

import '../../../../data/services/database/database.dart';
import '../../../contacts/common/index.dart';
import '../../../leads/common/search.dart';

class ContactLeadSelect extends StatefulWidget {
  const ContactLeadSelect({
    Key key,
    @required this.contactsChanged,
    @required this.leadsChanged,
    @required this.contacts,
    @required this.leads,
  }) : super(key: key);
  final ValueChanged<List<Contact>> contactsChanged;
  final ValueChanged<List<Lead>> leadsChanged;
  final List<Contact> contacts;
  final List<Lead> leads;

  @override
  _ContactLeadSelectState createState() => _ContactLeadSelectState();
}

class _ContactLeadSelectState extends State<ContactLeadSelect> {
  List<Contact> _contacts;
  List<Lead> _leads;

  @override
  void initState() {
    _contacts = widget.contacts;
    _leads = widget.leads;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: <Widget>[
        ExpansionTile(
          title: Text('Contacts'),
          leading: IconButton(
            icon: Icon(Icons.search),
            onPressed: () {
              Navigator.of(context)
                  .push<List<Contact>>(MaterialPageRoute(
                builder: (_) => SearchContacts(
                  selected: _contacts,
                ),
              ))
                  .then((items) {
                if (items != null) {
                  if (mounted)
                    setState(() {
                      _contacts = items;
                    });
                  widget.contactsChanged(_contacts);
                }
              });
            },
          ),
          children: <Widget>[
            if (_contacts == null) ...[
              ListTile(
                title: Text('No Contacts Selected'),
              ),
            ] else ...[
              for (var item in _contacts) ...[
                ListTile(
                  leading: Icon(Icons.person),
                  title: Text(item.firstName),
                  subtitle: Text(item.lastActivity),
                  trailing: IconButton(
                    icon: Icon(Icons.close),
                    onPressed: () {
                      if (mounted)
                        setState(() {
                          _contacts.remove(item);
                        });
                      widget.contactsChanged(_contacts);
                    },
                  ),
                ),
              ],
            ],
          ],
        ),
        ExpansionTile(
          title: Text('Leads'),
          leading: IconButton(
            icon: Icon(Icons.search),
            onPressed: () {
              Navigator.of(context)
                  .push<List<Lead>>(MaterialPageRoute(
                builder: (_) => SearchLeads(
                  selected: _leads,
                ),
              ))
                  .then((items) {
                if (items != null) {
                  if (mounted)
                    setState(() {
                      _leads = items;
                    });
                  widget.leadsChanged(_leads);
                }
              });
            },
          ),
          children: <Widget>[
            if (_leads == null) ...[
              ListTile(
                title: Text('No Leads Selected'),
              ),
            ] else ...[
              for (var item in _leads) ...[
                ListTile(
                  leading: Icon(Icons.person),
                  title: Text(item.firstName),
                  subtitle: Text(item.lastActivity),
                  trailing: IconButton(
                    icon: Icon(Icons.close),
                    onPressed: () {
                      if (mounted)
                        setState(() {
                          _leads.remove(item);
                        });
                      widget.leadsChanged(_leads);
                    },
                  ),
                ),
              ],
            ],
          ],
        ),
      ],
    );
  }
}
