part of unify;

class LeadDetails implements UnifyObject {
  LeadDetails(this.client, this.json, this.id);

  @override
  final UnifyClient client;

  @override
  final Map<String, dynamic> json;

  final String id;

  String get fullName => "$firstName $lastName".trim();

  String get displayName => "$lastName, $firstName";

  String get firstName => json["First_Name"];

  String get middleName => json["Middle_Name"];

  String get lastName => json["Last_Name"];

  String get suffix => json["Suffix"];

  String get lastActivity => json["Last_Activity"];

  String get secondaryFirstName => json["Secondary_First_Name"];

  String get secondaryMiddleName => json["Secondary_Middle_Name"];

  String get secondaryLastName => json["Secondary_Last_Name"];

  PhoneNumber get cellPhone => PhoneNumber(client, json["Cell_Phone"]);

  PhoneNumber get homePhone => PhoneNumber(client, json["Home_Phone"]);

  PhoneNumber get officePhone => PhoneNumber(client, json["Office_Phone"]);

  PhoneNumber get faxPhone => PhoneNumber(client, json["Fax_Phone"]);

  PhoneNumber get secondaryCellPhone =>
      PhoneNumber(client, json["Secondary_Cell_Phone"]);

  String get email => json["Email_Address"];

  DateTime get dateCreated => DateTime.tryParse(json["Date_Created"] as String);

  DateTime get dateModified =>
      DateTime.tryParse(json["Date_Modified"] as String);

  PostalAddress get currentAddress =>
      PostalAddress(client, json["Current_Address"]);

  PostalAddress get propertyAddress =>
      PostalAddress(client, json["Property_Address"]);

  List<LeadGroup> get groups => json["Lead_Groups"] == null
      ? []
      : List?.from(json["Lead_Groups"])
              ?.map((i) => LeadGroup(client, i))
              ?.toList() ??
          [];

  bool get recentBankruptcy => json["Recent_Bankruptcy"];

  String get creditRange => json["Credit_Range"];

  String get secondaryCreditRange => json["Secondary_Credit_Range"];

  double get annualGrossIncome => json["Annual_Gross_Income"];

  double get propertyValue => json["Property_Value"];

  String get bestCallTime => json["Best_Call_Time"];

  LoanDetails get currentLoan =>
      LoanDetails(client, json["Current_Loan"], null);

  LoanDetails get secondLoan => LoanDetails(client, json["Second_Loan"], null);

  LoanDetails get requestedLoan =>
      LoanDetails(client, json["Requested_Loan"], null);

  List<Tracker> get tracker => json["Lead_Tracker"] == null
      ? []
      : List?.from(json["Lead_Tracker"])
              ?.map((i) => Tracker(client, i))
              ?.toList() ??
          [];

  List<LeadNote> get notes => json["Lead_Notes"] == null
      ? []
      : List?.from(json["Lead_Notes"])
              ?.map((i) => LeadNote(client, i))
              ?.toList() ??
          [];

  List<Task> get tasks => json["Lead_Tasks"] == null
      ? []
      : List?.from(json["Lead_Tasks"])?.map((i) => Task(client, i))?.toList() ??
          [];

  List<PhoneNumber> get numbers {
    List<PhoneNumber> _items = [];
    if (cellPhone != null && cellPhone.raw().isNotEmpty) _items.add(cellPhone);
    if (homePhone != null && homePhone.raw().isNotEmpty) _items.add(homePhone);
    if (officePhone != null && officePhone.raw().isNotEmpty)
      _items.add(officePhone);
    return _items ?? [];
  }

  // -- Actions --
  Future<LeadDetails> getDetails() {
    return client.getLeadDetails(id);
  }

  Future<List<LeadNote>> listNotes() {
    return client.listLeadNotes(id);
  }

  Future<List<Tracker>> listTracker() {
    return client.listLeadTracker(id);
  }

  Future<List<Task>> listTasks({DateTime date}) {
    return client.listTasksForLead(id, date: date);
  }

  Future<List<String>> listResponses() {
    return client.listLeadLogResponses();
  }

  Future addNote(String text) {
    return client.addLeadNote(id, text);
  }

  Future addLogResponse({@required String action, String comment}) {
    return client.addLeadLogResponse(id, action: action, comment: comment);
  }

  Future delete() {
    return client.deleteLead(id);
  }

  /// type: `0 (Task)`, `1 (Appointment)`, `2 (Scheduled Call)`
  ///
  /// alertType: `Email`, `TextMessage`, `Email, TextMessage`, ""
  ///
  /// alertOffset: int for `minutes`
  Future addFollowUp({
    @required String title,
    DateTime date,
    int type = 0,
    String description = "",
    bool dismissOnChange = true,
    String alertType,
    int alertOffset = 0,
    String userNumber,
    String prospectNumber,
  }) {
    return client.addLeadFollowUp(
      id,
      title: title,
      date: date,
      type: type,
      description: description,
      dismissOnChange: dismissOnChange,
      alertType: alertType,
      alertOffset: alertOffset,
      userNumber: userNumber,
      prospectNumber: prospectNumber,
    );
  }

  Future edit({
    @required String firstName,
    @required String lastName,
    String middleName,
    String suffix,
    String secondaryFirstName,
    String secondaryMiddleName,
    String secondaryLastName,
    PostalAddress currentAddress,
    PostalAddress propertyAddress,
    PhoneNumber cellPhone,
    PhoneNumber homePhone,
    PhoneNumber officePhone,
    PhoneNumber faxPhone,
    PhoneNumber secondaryCellPhone,
    String email,
    List<LeadGroup> groups,
    String bestCallTime,
    String creditRange,
    String secondaryCreditRange,
    double annualGrossIncome,
    bool recentBankruptcy,
    double propertyValue,
  }) {
    return client.editLead(
      id,
      firstName: firstName,
      lastName: lastName,
      middleName: middleName,
      suffix: suffix,
      secondaryFirstName: secondaryFirstName,
      secondaryMiddleName: secondaryMiddleName,
      secondaryLastName: secondaryLastName,
      currentAddress: currentAddress,
      cellPhone: cellPhone,
      homePhone: homePhone,
      officePhone: officePhone,
      faxPhone: faxPhone,
      secondaryCellPhone: secondaryCellPhone,
      email: email,
      groups: groups,
      propertyAddress: propertyAddress,
      bestCallTime: bestCallTime,
      annualGrossIncome: annualGrossIncome,
      recentBankruptcy: recentBankruptcy,
      propertyValue: propertyValue,
      secondaryCreditRange: secondaryCreditRange,
    );
  }
}
