part of unify;

class LoanDetails implements UnifyObject {
  LoanDetails(this.client, this.json, this.id);

  @override
  final UnifyClient client;

  @override
  final Map<String, dynamic> json;

  final String id;

  String get contactId => json["contact_id"];

  String get firstName => json["borrower_first_name"];

  String get middleName => json["borrower_middle_name"];

  String get lastName => json["borrower_last_name"];

  String get fullName => "$firstName $middleName $lastName".trim();

  String get displayName => "$lastName, $firstName";

  String get coFristName => json["coborrower_first_name"];

  String get coMiddleName => json["coborrower_middle_name"];

  String get coLastName => json["coborrower_last_name"];

  String get coFullName => "$coFristName $coMiddleName $coLastName".trim();

  String get coDisplayName => "$coLastName, $coFristName";

  num get purchasePrice => json["purchase_price"];

  num get loanAmount => json["loan_amount"];

  num get estimatedValue => json["estimated_value"];

  num get appraisedValue => json["appraised_value"];

  num get lvt => json["lvt"];

  bool get escrowWaived => json["escrow_waived"];

  PostalAddress get propertyAddress => PostalAddress(client, json["property_address"]);

  PostalAddress get currentAddress => PostalAddress(client, json["current_address"]);

  DateTime get dateClosed => DateTime.tryParse(json["date_closed"]);

  DateTime get dateApplication => DateTime.tryParse(json["date_application"]);

  DateTime get dateEstimatedClosing => DateTime.tryParse(json["date_estimated_closing"]);

  DateTime get dateFirstPayment => DateTime.tryParse(json["date_first_payment"]);

  String get loanPurpose => json["loan_purpose"];

  String get programTitle => json["program_title"];

  String get lockStatus => json["lock_status"];

  DateTime get lockDate => DateTime.tryParse(json["lock_date"]);

  DateTime get lockExpiration => DateTime.tryParse(json["lock_expiration"]);

  num get rate => json["rate"];

  num get apr => json["apr"];

  String get amortization => json["amortization"];

  num get term => json["term"];

  num get piPayment => json["pi_payment"];

  num get hazard => json["hazard"];

  num get taxes => json["taxes"];

  num get pmi => json["pmi"];

  num get hoa => json["hoa"];

  num get otherPayment => json["other_payment"];

  num get totalPayment => json["total_payment"];

  String get loanNumber => json["loan_number"];

  String get lenderName => json["lender_name"];

  num get loanBalance => json["loan_balance"];

  String get loanType => json["loan_type"];

  LoanProgress get progress => LoanProgress(client, json["loan_progress"]);

  List<LoanContact> get contacts => json["loan_contacts"] == null
      ? []
      : List?.from(json["loan_contacts"])
              ?.map((i) => LoanContact(client, i))
              ?.toList() ??
          [];

  // -- Actions --
  Future<List<LoanContact>> listContacts() {
    if (id == null) throw "Loan ID Not Found";
    return client.listLoanContacts(id);
  }

  Future<LoanProgress> getProgress() {
    if (id == null) throw "Loan ID Not Found";
    return client.getLoanStatus(id);
  }

  Future<ContactDetails> getBorrower() {
    return client.getContactDetails(contactId);
  }
}


